// use launchy::launch_control::Message::Release;
use crate::Button::GridButton;
use crate::Message::Press;
use crate::Message::Release;
use enigo::Enigo;
use enigo::Key;
use enigo::KeyboardControllable;
use launchy::launchpad_s::*;
use launchy::InputDevice;
use launchy::MsgPollingWrapper;
use launchy::OutputDevice;
// use launchy::{Canvas as _, CanvasMessage, Color, MsgPollingWrapper as _};
#[derive(Copy, Clone)]
enum Action<'a> {
    TypeChar(char),
    TypeString(&'a str),
    RunCommand(&'a str),
}

fn main() {
    // println!("Reading config");
    let pad_action_array: [Action; 80] = [Action::TypeChar('a'); 80];
    let mut enigo = Enigo::new();

    let input = Input::guess_polling().unwrap();
    let mut output = Output::guess().unwrap();

    for x in 0..10 {
        let color = match x / 2 == 1 {
            true => Color::RED,
            false => Color::AMBER,
        };
        let button = Button::ControlButton { index: x };

        output.light(button, color).unwrap();
    }

    for x in 0..9 {
        for y in 0..8 {
            let color = Color::GREEN;
            let button = Button::GridButton { x, y };

            output.light(button, color).unwrap();
        }
    }

    for msg in input.iter() {
        // canvas[] =

        match msg {
            Press { button } => match button {
                Button::ControlButton { index } => match index {
                    0 => {
                        enigo.key_down(Key::UpArrow);
                    }
                    1 => {
                        enigo.key_down(Key::DownArrow);
                    }
                    2 => {
                        enigo.key_down(Key::LeftArrow);
                    }
                    3 => {
                        enigo.key_down(Key::RightArrow);
                    }
                    4 => {
                        println!("hi")
                    }

                    _ => {}
                },

                GridButton { x, y } => match (x, y) {
                    (x, 0) => match x {
                        0 => {
                            enigo.key_down(Key::F12);
                        }
                        1 => {
                            enigo.key_down(Key::F13);
                        }
                        2 => {
                            enigo.key_down(Key::F14);
                        }
                        3 => {
                            enigo.key_down(Key::F15);
                        }
                        4 => {
                            enigo.key_down(Key::F16);
                        }
                        5 => {
                            enigo.key_down(Key::F17);
                        }
                        6 => {
                            enigo.key_down(Key::F18);
                        }
                        7 => {
                            enigo.key_down(Key::F19);
                        }

                        _ => {}
                    },
                    (0, 7) => {
                        enigo.key_down(Key::Control);
                        enigo.key_down(Key::Alt);
                        enigo.key_down(Key::Layout('m'));
                    }
                    (1, 7) => {
                        enigo.key_down(Key::Control);
                        enigo.key_down(Key::Alt);
                        enigo.key_down(Key::Layout('d'));
                    }

                    (8, 7) => {
                        enigo = type_string(
                            enigo,
                            "https://web.mit.edu/~simsong/www/ugh.pdf".to_string(),
                        );
                    }

                    (x, y) => {
                        println!("x{} y{}", x, y);
                    }
                },
            },
            Release { button } => match button {
                Button::ControlButton { index } => match index {
                    0 => {
                        enigo.key_up(Key::UpArrow);
                    }
                    1 => {
                        enigo.key_up(Key::DownArrow);
                    }
                    2 => {
                        enigo.key_up(Key::LeftArrow);
                    }
                    3 => {
                        enigo.key_up(Key::RightArrow);
                    }
                    _ => {}
                },
                GridButton { x, y } => match (x, y) {
                    (x, 0) => match x {
                        0 => {
                            enigo.key_up(Key::F12);
                        }
                        1 => {
                            enigo.key_up(Key::F13);
                        }
                        2 => {
                            enigo.key_up(Key::F14);
                        }
                        3 => {
                            enigo.key_up(Key::F15);
                        }
                        4 => {
                            enigo.key_up(Key::F16);
                        }
                        5 => {
                            enigo.key_up(Key::F17);
                        }
                        6 => {
                            enigo.key_up(Key::F18);
                        }
                        7 => {
                            enigo.key_up(Key::F19);
                        }

                        _ => {}
                    },
                    (0, 7) => {
                        enigo.key_up(Key::Control);
                        enigo.key_up(Key::Alt);
                        enigo.key_up(Key::Layout('m'));
                    }
                    (1, 7) => {
                        enigo.key_up(Key::Control);
                        enigo.key_up(Key::Alt);
                        enigo.key_up(Key::Layout('d'));
                    }
                    (_, _) => {}
                },
            },
            _ => {
                println!("{:?}", msg);
            }
        }
    }
}
// Used like this because of the style of output it does :)
// you could use  enigo.key_sequence(sequence) for speed
fn type_string(mut enigo: Enigo, string: String) -> Enigo {
    for c in string.chars() {
        enigo.key_down(Key::Layout(c));
        enigo.key_up(Key::Layout(c));
    }
    enigo
}
